getSF2009.5 <- function(summary.table, geo.filter, state, zip.file, type="e", dictionary=FALSE){
	
	if(type!="e" & type!="m") stop('type must be either "e" or "m".')
	# Load the data dictionaries
#	load('acs2009_5_data_dict.Rdata')
	load(paste(system.file(package="UScensus"),'/data/acs2009_5_data_dict.Rdata',sep=''))
	
	#unzip the geoheader file.
	# The geoheaderfile is called 'g20095XX.txt', where XX is the 2-letter statecode (or 'us')
	geo.file.name <- paste('g20095',state,'.txt',sep='')
	
	# Create a file connection to the geoheader
	geo.header.con <- unz(zip.file,geo.file.name)
	# Read in the geo header (as fixed width file, the widths come from the geo.dict)
	geo.header <- read.fwf(geo.header.con, width=sf2009.5.geo.dict[,'Field.Size'], col.names=sf2009.5.geo.dict[,'Data.Dictionary.Reference.Name'],colClasses='character')
	
	
	# Filter the geoheader file.  
	# The geo.filter will have an unknown number of filter criteria.  I will initialize a filter vector of 1s, 
	# and for each new filter criteria, multiply the new 0-1 filter by the saved filter vector
	match.product <- rep(1,length=nrow(geo.header))
	for(i in 1:length(geo.filter)){
		match.product <- match.product*match(geo.header[,as.character(names(geo.filter)[i])], geo.filter[[i]], nomatch=0)
		}
	select.rows <- which(match.product>0)
	
	#Check if any summary.tables don't exist, and print warning.
	if(any(match(summary.table, sf2009.5.data.dict$table.id, nomatch=0)==0)){
		warning.text <- cat('Warning: Table(s) ', summary.table[(summary.table %in% sf2009.5.data.dict$table.id)==FALSE], ' were not found in the dictionary.\n')
		cat(warning.text)
#		warning(warning.text,immediate.=TRUE)
		}
	
	# Now filter the data dictionary
	dict.rows <- which(match(sf2009.5.data.dict$table.id, summary.table, nomatch=0)>0)
	
	
	# Save the filtered data dictionary
	this.dictionary <- sf2009.5.data.dict[dict.rows,]
	# Extract the unique file numbers we will need
	file.numbers <- unique(this.dictionary$seq)
	
	# Create the output dataframe
	data.out <- geo.header[select.rows,]
	
	# Set the LOGRECNO as the rowname
	row.names(data.out) <- data.out$LOGRECNO
	
	# For each file
	for(i in 1:length(file.numbers)){
		# unzip the csv file
		file.name <- paste(type,'20095',state,file.numbers[i],'000.txt',sep='') #Create the filename
		data.file.con <- unz(zip.file, file=file.name) # Create the file connection
		
		# Read in the data
		this.file <- try(read.table(data.file.con,sep=',',colClasses='character'),silent=TRUE)
		if(class(this.file)=='try-error'){
			cat('An error was caught when opening ', file.name,' Chances are that the file is empty for this state\n', this.file)
			next
			}
		# The logrecno is in column 6.... make that the row name
		row.names(this.file) <- this.file$V6
		
		# Filter this.file based on select.rows  DELETED BECAUSE MERGE() IS SAFER
#		this.file <- this.file[select.rows,drop=FALSE]
		
		
		#Filter the columns based on sf2009.5.data.dict
		# First, find the subset of the data dictionary for this file...
		sub.this.dictionary <- this.dictionary[which(this.dictionary$seq==file.numbers[i]),]
		this.file <- this.file[,sub.this.dictionary$column,drop=FALSE]
		
		# Give field names to this.file
		names(this.file) <- sub.this.dictionary$field.id
		
#		data.out <- cbind(data.out, this.file)

		data.out <- merge(data.out, this.file, by.x='row.names',by.y='row.names',all.x=TRUE)
		row.names(data.out) <- data.out$LOGRECNO
		# By default, merge add the row.names as columns.  We don't want that.
		data.out <- data.out[,-1,drop=FALSE]
		
		if(dictionary==TRUE){
			if(i==1){dict.out <- sub.this.dictionary}
			else if(i>1) dict.out <- rbind(dict.out, sub.this.dictionary)
			}
		}
		
		# Create return value(s)
		if(dictionary==TRUE) return(list(data=data.out,dictionary=dict.out))
		return(data.out)
	}