# make SF2009_5 data dictionary

working.dir <- getwd()

# Load the data dictionary provided by the Census Bureau.
# This was an excel file 'Sequence_Number_and_Table_Number_Lookup.xls'
#  that I saved as a .csv

data.dict.raw <- read.csv('Sequence_Number_and_Table_Number_Lookup.csv')
# data.dict.raw has 
# File.ID: something like ACSSF
# Table.ID something like B07401
# Sequence.Number: the file number.  1,2,3,...
# Line.Number: Location within the file
# Start.Position: Start of the Table
# Total Cells

# Now we want to dupicate the table name and Universe for each of its fields.
TableName <- vector(length=nrow(data.dict.raw)) #Initialize vector for the Table's Name
Universe <- vector(length=nrow(data.dict.raw)) #Initialize vector for the Table's Universe
for (i in 1:nrow(data.dict.raw)) {
	# The file format has numbers in Start.Position only on the first row of each table.  This is how we can find the first row of each table
	# we will walk down the file, and if we find a filled a nonempty Start Position, we start copying the Table Name and Universe until we find the next start position
	TableName[i]<-ifelse(is.na(data.dict.raw$Start.Position[i]),as.character(TableName[i-1]),as.character(data.dict.raw$Table.Title[i]))
	Universe[i]<-ifelse(is.na(data.dict.raw$Start.Position[i]),as.character(Universe[i-1]),as.character(data.dict.raw$Table.Title[i+1]))
	}

# Save these fields to data.dict.raw	
data.dict.raw$TableName <- TableName
data.dict.raw$Universe <- Universe

# Create a columns with the index for the first entry of each table.
# This is in column "Start.Position", but it needs to be copied into empty cells
# i.e. it is in the form 7 _ _ _ _ _ _ _ 15 _ _ _
#     and needs to be in 7 7 7 7 7 7 7 7 15 15 15 15 etc.
for(i in 2:nrow(data.dict.raw)) data.dict.raw$Start.Position[i]<-ifelse(is.na(data.dict.raw$Start.Position[i]),data.dict.raw$Start.Position[i-1],data.dict.raw$Start.Position[i])

# Remove the descriptive rows for each Table which are identified by a blank "Line.Number" (these don't correspond to columns in the data file)
data.dict.raw <- data.dict.raw[which(!is.na(data.dict.raw$Line.Number)),]

# Remove the descriptive rows for each Table which are not integer (these don't correspond to columns in the datafile either)
# nonintegers are found with which(data.dict.raw$Line.Number%%1>0)
data.dict.raw <- data.dict.raw[-which(data.dict.raw$Line.Number%%1>0),]

# Save the Table ID
Table=data.dict.raw$Table.ID
# Save the Table Descriptive Name
TableName <- data.dict.raw$TableName
# Save the Field ID
Field <- paste(data.dict.raw$Table.ID, sprintf('%03i',as.integer(data.dict.raw$Line.Number)),sep='')
# Save the Descriptive Field Name
FieldDes <- as.character(data.dict.raw$Table.Title)
# Save the file number
FileNo <- sprintf('%04i',data.dict.raw$Sequence.Number)
# Save the column number of the field
column <- data.dict.raw$Start.Position + as.integer(data.dict.raw$Line.Number) - 1

Universe <- data.dict.raw$Universe

sf2009.5.data.dict <- data.frame(table.id=Table, table.description=TableName, field.id=Field, field.description=FieldDes, seq=FileNo, column=column,table.universe=Universe)


# Read in the geographic data dictionary
# Landon Bevier made the geographic file by hand from the data documentation.
sf2009.5.geo.dict <- read.csv('acs2009_5_geo_data_dict.csv')

#Create Data Documentation
promptData('sf2009.5.data.dict')
promptData('sf2009.5.geo.dict')

save(sf2009.5.data.dict, sf2009.5.geo.dict, file='acs2009_5_data_dict.Rdata')

# When this is done, the data file 'acs2009_5_data_dict.Rdata' will need to be saved to the data directory of the pacage and the docs must be added to the man directory.


